from __future__ import absolute_import, unicode_literals

from django.conf.urls import url

from wagtaillinkchecker import views

urlpatterns = [
    url(r'^$', views.index,
        name='wagtaillinkchecker'),
    url(r'^settings/$', views.settings,
        name='wagtaillinkchecker_settings'),
    url(r'^scan/$', views.run_scan,
        name='wagtaillinkchecker_runscan'),
    url(r'^scan/(?P<scan_pk>\d+)/$', views.scan,
        name='wagtaillinkchecker_scan'),
    url(r'^scan/(?P<scan_pk>\d+)/delete$', views.delete,
        name='wagtaillinkchecker_delete'),
]



from __future__ import print_function

from django.shortcuts import get_object_or_404
from django.shortcuts import redirect, render

try:
    from django.utils.lru_cache import lru_cache
except ModuleNotFoundError:
    from functools import lru_cache

from django.utils.translation import ugettext_lazy as _

from wagtaillinkchecker.forms import SitePreferencesForm
from wagtaillinkchecker.models import SitePreferences, Scan
from wagtaillinkchecker.pagination import paginate
from wagtaillinkchecker.scanner import broken_link_scan, get_celery_worker_status
from wagtaillinkchecker import utils

if utils.is_wagtail_version_more_than_equal_to_2_0():
    from wagtail.admin import messages
    from wagtail.admin.edit_handlers import (ObjectList,
                                             extract_panel_definitions_from_model_class)
    from wagtail.core.models import Site
else:
    from wagtail.wagtailadmin import messages
    from wagtail.wagtailadmin.edit_handlers import (ObjectList,
                                                    extract_panel_definitions_from_model_class)
    from wagtail.wagtailcore.models import Site


@lru_cache()
def get_edit_handler(model):
    panels = extract_panel_definitions_from_model_class(model, ['site'])

    if utils.is_wagtail_version_more_than_equal_to_2_5():
        return ObjectList(panels).bind_to(model=model)
    else:
        return ObjectList(panels).bind_to_model(model)


def scan(request, scan_pk):
    scan = get_object_or_404(Scan, pk=scan_pk)

    return render(request, 'wagtaillinkchecker/scan.html', {
        'scan': scan
    })


def index(request):
    from django.conf import settings

    site = Site.find_for_request(request)
    scans = Scan.objects.filter(site=site).order_by('-scan_started')

    paginator, page = paginate(request, scans)

    return render(request, 'wagtaillinkchecker/index.html', {
        'page': page,
        'paginator': paginator,
        'scans': scans
    })


def delete(request, scan_pk):
    scan = get_object_or_404(Scan, pk=scan_pk)

    if request.method == 'POST':
        scan.delete()
        messages.success(request, _(
            'The scan results were successfully deleted.'))
        return redirect('wagtaillinkchecker')

    return render(request, 'wagtaillinkchecker/delete.html', {
        'scan': scan,
    })


def settings(request):
    site = Site.find_for_request(request)
    instance, created = SitePreferences.objects.get_or_create(site=site)
    form = SitePreferencesForm(instance=instance)
    form.instance.site = site
    object_list = get_edit_handler(SitePreferences)

    if request.method == "POST":
        instance = SitePreferences.objects.filter(site=site).first()
        form = SitePreferencesForm(request.POST, instance=instance)
        if form.is_valid():
            form.save()
            messages.success(request, _(
                'Link checker settings have been updated.'))
            return redirect('wagtaillinkchecker_settings')
        else:
            messages.error(request, _(
                'The form could not be saved due to validation errors'))
    else:
        form = SitePreferencesForm(instance=instance)
        if utils.is_wagtail_version_more_than_equal_to_2_5():
            edit_handler = object_list.bind_to(
                instance=SitePreferences, form=form, request=request)
        else:
            edit_handler = object_list.bind_to_instance(
                instance=SitePreferences, form=form, request=request)

    return render(request, 'wagtaillinkchecker/settings.html', {
        'form': form,
        'edit_handler': edit_handler,
    })


def run_scan(request):
    site = Site.find_for_request(request)
    celery_status = get_celery_worker_status()
    if 'ERROR' not in celery_status:
        broken_link_scan(site)
    else:
        messages.warning(request, _(
            'No celery workers are running, the scan was not conducted.'))

    return redirect('wagtaillinkchecker')



# Generated by Django 2.0.8 on 2018-09-22 22:35

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wagtaillinkchecker', '0004_auto_20180829_1922'),
    ]

    operations = [
        migrations.AlterField(
            model_name='scanlink',
            name='url',
            field=models.URLField(max_length=500),
        ),
    ]



# Generated by Django 2.0.5 on 2018-08-29 19:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wagtaillinkchecker', '0003_store_scan_results'),
    ]

    operations = [
        migrations.AlterField(
            model_name='scanlink',
            name='page_slug',
            field=models.CharField(blank=True, max_length=512, null=True),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2017-06-28 09:08
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailcore', '0033_remove_golive_expiry_help_text'),
        ('wagtaillinkchecker', '0002_auto_20160209_1533'),
    ]

    operations = [
        migrations.CreateModel(
            name='Scan',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('scan_finished', models.DateTimeField(blank=True, null=True)),
                ('scan_started', models.DateTimeField(auto_now_add=True)),
                ('site', models.ForeignKey(editable=False, on_delete=django.db.models.deletion.CASCADE, to='wagtailcore.Site')),
            ],
        ),
        migrations.CreateModel(
            name='ScanLink',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('url', models.URLField()),
                ('crawled', models.BooleanField(default=False)),
                ('invalid', models.BooleanField(default=False)),
                ('broken', models.BooleanField(default=False)),
                ('status_code', models.IntegerField(blank=True, null=True)),
                ('error_text', models.TextField(blank=True, null=True)),
                ('page_deleted', models.BooleanField(default=False)),
                ('page_slug', models.CharField(blank=True, max_length=128, null=True)),
                ('page', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='wagtailcore.Page')),
                ('scan', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='links', to='wagtaillinkchecker.Scan')),
            ],
        ),
        migrations.AlterField(
            model_name='sitepreferences',
            name='automated_scanning',
            field=models.BooleanField(default=False, help_text='Conduct automated sitewide scans for broken links, and send emails if a problem is found.', verbose_name='Automated Scanning'),
        ),
        migrations.AlterUniqueTogether(
            name='scanlink',
            unique_together=set([('url', 'scan')]),
        ),
    ]



from django import forms

from wagtaillinkchecker.models import SitePreferences


class SitePreferencesForm(forms.ModelForm):

    class Meta:
        model = SitePreferences
        exclude = ('site', )



import re
from wagtail import __version__ as WAGTAIL_VERSION


def is_wagtail_version_more_than_equal_to_2_5():
	expression = '^((2.([5-9]{1,}|([1-9]{1,}[0-9]{1,}))(.\d+)*)|(([3-9]{1,})(.\d+)*))$'

	return re.search(expression, WAGTAIL_VERSION)


def is_wagtail_version_more_than_equal_to_2_0():
	expression = '^((2.([0-9]{1,}|([1-9]{1,}[0-9]{1,}))(.\d+)*)|(([3-9]{1,})(.\d+)*))$'

	return re.search(expression, WAGTAIL_VERSION)

